/*---------------------------------------------------------------------\
|                          ____ _   __ __ ___                          |
|                         |__  / \ / / . \ . \                         |
|                           / / \ V /|  _/  _/                         |
|                          / /__ | | | | | |                           |
|                         /_____||_| |_| |_|                           |
|                                                                      |
\---------------------------------------------------------------------*/
/** \file	zypp-core/APIConfig.h
 *  \brief	Provides API related macros.
 */
#ifndef ZYPP_APICONFIG_H
#define ZYPP_APICONFIG_H
#define APIConfig(VAR) VAR

#define LIBZYPP_VERSION_STRING "17.38.8"
#define LIBZYPP_VERSION_MAJOR  17
#define LIBZYPP_VERSION_MINOR  38
#define LIBZYPP_VERSION_PATCH  8
#define LIBZYPP_SOVERSION      1735
#define LIBZYPP_VERSION        173808

#define LIBZYPP_CODESTREAM     ""
#define LIBZYPP_ZYPPCONFDIR    "/usr/etc"

// Legacy default behavior which might be enabled for legacy codestreams
#define LIBZYPP_CONFIG_USE_DELTARPM_BY_DEFAULT 0
#define LIBZYPP_CONFIG_USE_CLASSIC_RPMTRANS_BY_DEFAULT 0
#define LIBZYPP_CONFIG_USE_LEGACY_CURL_BACKEND_BY_DEFAULT 0
#define LIBZYPP_CONFIG_USE_SERIAL_PACKAGE_DOWNLOAD_BY_DEFAULT 0

#endif //ZYPP_APICONFIG_H
